@@ -8,7 +8,6 @@ use vb_accumulator::error::VBAccumulatorError;
 
 #[derive(Debug)]
 pub enum ProofSystemError {
-    OnlyOneMetaStatementSupportedForNow,
     UnequalWitnessAndStatementCount(usize, usize),
     WitnessIncompatibleWithStatement(usize, String, String),
     ProofIncompatibleWithStatement(usize, String, String),
@@ -16,12 +15,14 @@ pub enum ProofSystemError {
     BBSPlusProtocolMessageAbsent(usize, usize),
     SubProtocolNotReadyToGenerateChallenge(usize),
     SubProtocolAlreadyInitialized(usize),
-    SubProtocolNotReadyToGenerateProof(String),
+    SubProtocolNotReadyToGenerateProof(usize),
+    /// This error indicates that some witnesses that were required to be equal are not equal
     WitnessResponseNotEqual(usize, usize),
     Serialization(SerializationError),
     SchnorrError(SchnorrError),
     BBSPlusError(BBSPlusError),
     VBAccumError(VBAccumulatorError),
+    InvalidProofSpec,
 }
 
 impl From<SchnorrError> for ProofSystemError {
